package net.wanji.opt.service.impl;

import lombok.extern.slf4j.Slf4j;
import net.wanji.common.enums.BaseEnum;
import net.wanji.common.enums.TurnConvertEnum;
import net.wanji.databus.dao.entity.BaseCrossDirInfoPO;
import net.wanji.databus.dao.mapper.BaseCrossDirInfoMapper;
import net.wanji.databus.dao.mapper.BaseCrossTurnInfoMapper;
import net.wanji.databus.dao.mapper.CrossBaseLaneInfoMapper;
import net.wanji.databus.dao.mapper.LaneInfoMapper;
import net.wanji.databus.po.CrossTurnInfoPO;
import net.wanji.databus.po.LaneInfoPO;
import net.wanji.opt.bo.BottomMenuBO;
import net.wanji.opt.service.MainlineEvaluateService;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @author Kent HAN
 * @date 2023/9/19 15:24
 */
@Slf4j
@Service
public class MainlineEvaluateServiceImpl implements MainlineEvaluateService {
    private final BaseCrossDirInfoMapper baseCrossDirInfoMapper;
    private final BaseCrossTurnInfoMapper baseCrossTurnInfoMapper;
    private final LaneInfoMapper laneInfoMapper;

    public MainlineEvaluateServiceImpl(@Qualifier("baseCrossDirInfoMapper") BaseCrossDirInfoMapper baseCrossDirInfoMapper,
                                       @Qualifier("baseCrossTurnInfoMapper") BaseCrossTurnInfoMapper baseCrossTurnInfoMapper,
                                       CrossBaseLaneInfoMapper crossBaseLaneInfoMapper, @Qualifier("laneInfoMapper") LaneInfoMapper laneInfoMapper) {
        this.baseCrossDirInfoMapper = baseCrossDirInfoMapper;
        this.baseCrossTurnInfoMapper = baseCrossTurnInfoMapper;
        this.laneInfoMapper = laneInfoMapper;
    }

    @Override
    public List<String> bottomMenu(BottomMenuBO bo) {
        String crossId = bo.getCrossId();
        Integer scope = bo.getScope();

        if (scope == 1) {
            return buildDirMenu(crossId);
        } else if (scope == 2) {
            return buildTurnMenu(crossId);
        } else if (scope == 3) {
            return buildLaneMenu(crossId);
        }

        return null;
    }

    private List<String> buildLaneMenu(String crossId) {
        List<BaseCrossDirInfoPO> poList = baseCrossDirInfoMapper.selectByCrossId(crossId);
        List<BaseCrossDirInfoPO> dirList = poList.stream()
                .filter(po -> po.getInOutType() == 1)
                .collect(Collectors.toList());
        List<String> res = new ArrayList<>();
        for (BaseCrossDirInfoPO dirPo : dirList) {
            Integer dirType = dirPo.getDirType();
            String dirName = BaseEnum.SignalDirectionEnum.getNameByCode(dirType);

            List<LaneInfoPO> laneList = laneInfoMapper.selectInDirByCrossIdAndDir(crossId, dirType);
            for (LaneInfoPO lanePO : laneList) {
                Integer turn = lanePO.getTurn();
                String turnDesc = TurnConvertEnum.getDescByKey(turn);
                Integer sort = lanePO.getSort();
                res.add(dirName + "进口" + turnDesc + sort + "车道");
            }
        }
        return res;
    }

    private List<String> buildTurnMenu(String crossId) {
        List<BaseCrossDirInfoPO> poList = baseCrossDirInfoMapper.selectByCrossId(crossId);
        List<BaseCrossDirInfoPO> dirList = poList.stream()
                .filter(po -> po.getInOutType() == 1)
                .collect(Collectors.toList());
        List<String> res = new ArrayList<>();
        for (BaseCrossDirInfoPO dirPo : dirList) {
            Integer dirType = dirPo.getDirType();
            String dirName = BaseEnum.SignalDirectionEnum.getNameByCode(dirType);

            List<CrossTurnInfoPO> turnList = baseCrossTurnInfoMapper.selectByCrossIdAndDir(crossId, dirType);
            for (CrossTurnInfoPO turnPO : turnList) {
                String turnType = turnPO.getTurnType();
                String turnDesc = TurnConvertEnum.getDescByCode(turnType);
                res.add(dirName + "进口" + turnDesc);
            }
        }
        return res;
    }

    private List<String> buildDirMenu(String crossId) {
        List<BaseCrossDirInfoPO> poList = baseCrossDirInfoMapper.selectByCrossId(crossId);
        List<BaseCrossDirInfoPO> filteredList = poList.stream()
                .filter(po -> po.getInOutType() == 1)
                .collect(Collectors.toList());
        List<String> res = new ArrayList<>();
        for (BaseCrossDirInfoPO po : filteredList) {
            Integer dirType = po.getDirType();
            String dirName = BaseEnum.SignalDirectionEnum.getNameByCode(dirType);
            res.add(dirName + "进口");
        }
        return res;
    }


}
